@@ -312,7 +312,7 @@ int tarobject(struct TarInfo *ti) {
     }
     break;
   case NormalFile0: case NormalFile1:
-  case CharacterDevice: case BlockDevice:
+  case CharacterDevice: case BlockDevice: case FIFO:
   case HardLink:
     break;
   default:
@@ -350,9 +350,15 @@ int tarobject(struct TarInfo *ti) {
           printf("Replacing files in old package %s ...\n",otherpkg->name);
           otherpkg->clientdata->replacingfilesandsaid= 1;
         } else {
-          forcibleerr(fc_overwrite,
-                      "trying to overwrite `%.250s', which is also in package %.250s",
-                      nifd->namenode->name,otherpkg->name);
+          if (S_ISDIR(stab.st_mode)) {
+            forcibleerr(fc_overwritedir, "trying to overwrite directory `%.250s' "
+                        "in package %.250s with nondirectory",
+                        nifd->namenode->name,otherpkg->name);
+          } else {
+            forcibleerr(fc_overwrite,
+                        "trying to overwrite `%.250s', which is also in package %.250s",
+                        nifd->namenode->name,otherpkg->name);
+          }
         }
       }
     }
@@ -405,6 +411,12 @@ int tarobject(struct TarInfo *ti) {
       ohshite("error closing/writing `%.255s'",ti->Name);
     newtarobject_utime(fnamenewvb.buf,ti);
     break;
+  case FIFO:
+    if (mkfifo(fnamenewvb.buf,ti->Mode & S_IFMT))
+      ohshite("error creating pipe `%.255s'",ti->Name);
+    debug(dbg_eachfiledetail,"tarobject FIFO");
+    newtarobject_allmodes(fnamenewvb.buf,ti);
+    break;
   case CharacterDevice: case BlockDevice:
     if (mknod(fnamenewvb.buf,ti->Mode & S_IFMT,ti->Device))
       ohshite("error creating device `%.255s'",ti->Name);
@@ -643,7 +655,7 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
 void cu_cidir(int argc, void **argv) {
   char *cidir= (char*)argv[0];
   char *cidirrest= (char*)argv[1];
-  *cidirrest= 0;
+  cidirrest[-1]= 0;
   ensure_pathname_nonexisting(cidir);
 }  
 
@@ -741,6 +753,7 @@ void archivefiles(const char *const *argv) {
   modstatdb_init(admindir,
                  f_noact ?                     msdbrw_readonly
                : cipaction->arg == act_avail ? msdbrw_write
+               : fc_nonroot ?                  msdbrw_write
                :                               msdbrw_needsuperuser);
 
   currenttime= time(0);
