@@ -36,7 +36,7 @@
 #include "filesdb.h"
 #include "main.h"
 
-static int listqcmp(const void *a, const void *b) {
+int pkglistqsortcmp(const void *a, const void *b) {
   struct pkginfo *pa= *(struct pkginfo**)a;
   struct pkginfo *pb= *(struct pkginfo**)b;
   return strcmp(pa->name,pb->name);
@@ -100,7 +100,7 @@ void listpackages(const char *const *argv) {
   iterpkgend(it);
   assert(i==np);
 
-  qsort(pkgl,np,sizeof(struct pkginfo*),listqcmp);
+  qsort(pkgl,np,sizeof(struct pkginfo*),pkglistqsortcmp);
   head=0;
   
   if (!*argv) {
@@ -305,6 +305,21 @@ static int searchoutput(struct filenamenode *namenode) {
   int found, i;
   struct filepackages *packageslump;
 
+  if (namenode->divert) {
+    for (i=0; i<2; i++) {
+      if (namenode->divert->pkg) printf("diversion by %s",namenode->divert->pkg->name);
+      else printf("local diversion");
+      printf(" %s: %s\n", i ? "to" : "from",
+             i ?
+             (namenode->divert->useinstead
+              ? namenode->divert->useinstead->name
+              : namenode->name)
+             :
+             (namenode->divert->camefrom
+              ? namenode->divert->camefrom->name
+              : namenode->name));
+    }
+  }
   found= 0;
   for (packageslump= namenode->packages;
        packageslump;
@@ -316,7 +331,7 @@ static int searchoutput(struct filenamenode *namenode) {
     }
   }
   if (found) printf(": %s\n",namenode->name);
-  return found;
+  return found + (namenode->divert ? 1 : 0);
 }
 
 void searchfiles(const char *const *argv) {
@@ -331,6 +346,7 @@ void searchfiles(const char *const *argv) {
 
   modstatdb_init(admindir,msdbrw_readonly);
   ensure_allinstfiles_available_quiet();
+  ensure_diversions();
 
   while ((thisarg= *argv++) != 0) {
     found= 0;
@@ -367,6 +383,7 @@ void enqperpackage(const char *const *argv) {
   const char *thisarg;
   struct fileinlist *file;
   struct pkginfo *pkg;
+  struct filenamenode *namenode;
   
   if (!*argv)
     badusage("--%s needs at least one package name argument", cipaction->olong);
@@ -411,12 +428,20 @@ void enqperpackage(const char *const *argv) {
         
       default:
         ensure_packagefiles_available(pkg);
+        ensure_diversions();
         file= pkg->clientdata->files;
         if (!file) {
           printf("Package `%s' does not contain any files (!)\n",pkg->name);
         } else {
           while (file) {
-            puts(file->namenode->name);
+            namenode= file->namenode;
+            puts(namenode->name);
+            if (namenode->divert && !namenode->divert->camefrom) {
+              if (!namenode->divert->pkg) printf("locally diverted");
+              else if (pkg == namenode->divert->pkg) printf("package diverts others");
+              else printf("diverted by %s",namenode->divert->pkg->name);
+              printf(" to: %s\n",namenode->divert->useinstead->name);
+            }
             file= file->next;
           }
         }
@@ -455,7 +480,7 @@ void assertpredep(const char *const *argv) {
   switch (pkg->status) {
   case stat_installed:
     break;
-  case stat_unpacked: case stat_halfconfigured:
+  case stat_unpacked: case stat_halfconfigured: case stat_halfinstalled:
     if (versionsatisfied3(&pkg->configversion,&predepversion,dvr_laterequal))
       break;
     printf("Version of dpkg with Pre-Depends support not yet configured.\n"
@@ -572,9 +597,9 @@ void printinstarch(const char *const *argv) {
 }
 
 void printarch(const char *const *argv) {
-  static const struct { const char *from, *to; } archtable[]= {
+  static const struct { const char *from, *to, *gnu; } archtable[]= {
 #include "archtable.inc"
-    { 0,0 }
+    { 0,0,0 }
   }, *archp;
                   
   const char *ccompiler, *arch;
@@ -613,7 +638,11 @@ void printarch(const char *const *argv) {
   for (archp=archtable;
        archp->from && !(strlen(archp->from) == ll && !strncmp(archp->from,p,ll));
        archp++);
-  arch= archp->to;
+  switch (cipaction->arg) {
+  case act_printarch:    arch= archp->to;  break;
+  case act_printgnuarch: arch= archp->gnu; break;
+  default: internerr("unknown action in printarch");
+  }
   if (!arch) {
     *q= 0; arch= p;
     fprintf(stderr,DPKG ": warning, architecture `%s' not in remapping table\n",arch);
@@ -627,7 +656,7 @@ void cmpversions(const char *const *argv) {
     const char *string;
     /* These values are exit status codes, so 0=true, 1=false */
     int if_lesser, if_equal, if_greater;
-    int if_none_a, if_none_b, if_none_both;
+    int if_none_a, if_none_both, if_none_b;
   };
 
   static const struct relationinfo relationinfos[]= {
@@ -638,15 +667,15 @@ void cmpversions(const char *const *argv) {
     { "ne",        0,1,0, 0,1,0  },
     { "ge",        1,0,0, 1,0,0  },
     { "gt",        1,1,0, 1,1,0  },
-    { "le-nl",     0,0,1, 1,0,0  }, /* Here none
-    { "lt-nl",     0,1,1, 1,1,0  },  * is counted
-    { "ge-nl",     1,0,0, 0,0,1  },  * than any version.
-    { "gt-nl",     1,1,0, 0,1,1  },  */
-    { "<",         0,0,1, 0,0,1  }, /* For compatibility
-    { "<=",        0,0,1, 0,0,1  },  * with dpkg
-    { "<<",        0,1,1, 0,1,1  },  * control file
-    { "=",         1,0,1, 1,0,1  },  * syntax
-    { ">",         1,0,0, 1,0,0  },  */
+    { "le-nl",     0,0,1, 1,0,0  }, /* Here none        */
+    { "lt-nl",     0,1,1, 1,1,0  }, /* is counted       */
+    { "ge-nl",     1,0,0, 0,0,1  }, /* than any version.*/
+    { "gt-nl",     1,1,0, 0,1,1  }, /*                  */
+    { "<",         0,0,1, 0,0,1  }, /* For compatibility*/
+    { "<=",        0,0,1, 0,0,1  }, /* with dpkg        */
+    { "<<",        0,1,1, 0,1,1  }, /* control file     */
+    { "=",         1,0,1, 1,0,1  }, /* syntax           */
+    { ">",         1,0,0, 1,0,0  }, /*                  */
     { ">=",        1,0,0, 1,0,0  },
     { ">>",        1,1,0, 1,1,0  },
     {  0                         }
