@@ -60,6 +60,8 @@ Usage: \n\
   " DPKG " -A|--record-avail <.deb file name> ... | -R|--recursive <dir> ...\n\
   " DPKG " --configure           <package name> ... | -a|--pending\n\
   " DPKG " -r|--remove | --purge <package name> ... | -a|--pending\n\
+  " DPKG " --get-selections [<pattern> ...]   get list of selections to stdout\n\
+  " DPKG " --set-selections                   set package selections from stdin\n\
   " DPKG " --update-avail <Packages-file>     replace available packages info\n\
   " DPKG " --merge-avail <Packages-file>      merge with info from file\n\
   " DPKG " --clear-avail                      erase existing available info\n\
@@ -71,6 +73,7 @@ Usage: \n\
   " DPKG " -S|--search <pattern> ...          find package(s) owning file(s)\n\
   " DPKG " -C|--audit                         check for broken package(s)\n\
   " DPKG " --print-architecture               print target architecture (uses GCC)\n\
+  " DPKG " --print-gnu-build-architecture     print GNU version of target arch\n\
   " DPKG " --print-installation-architecture  print host architecture (for inst'n)\n\
   " DPKG " --compare-versions <a> <rel> <b>   compare version numbers - see below\n\
   " DPKG " --help | --version                 show this help / version number\n\
@@ -99,7 +102,7 @@ Options:
 \n\
 Comparison operators for --compare-versions are:\n\
  lt le eq ne ge gt       (treat no version as earlier than any version);\n\
- lt-nl le-nl ge-nl gt-nl (tread no version as later than any version);\n\
+ lt-nl le-nl ge-nl gt-nl (treat no version as later than any version);\n\
  < << <= = >= >> >       (only for compatibility with control file syntax).\n\
 \n\
 Use `" DSELECT "' for user-friendly package management.\n",
@@ -125,6 +128,7 @@ unsigned long f_debug=0;
 int fc_downgrade=1, fc_configureany=0, fc_hold=0, fc_removereinstreq=0, fc_overwrite=1;
 int fc_removeessential=0, fc_conflicts=0, fc_depends=0, fc_dependsversion=0;
 int fc_autoselect=1, fc_badpath=0, fc_overwritediverted=0, fc_architecture=0;
+int fc_nonroot=0, fc_overwritedir=0;
 
 const char *admindir= ADMINDIR;
 const char *instdir= "";
@@ -144,8 +148,10 @@ static const struct forceinfo {
   { "depends-version",     &fc_dependsversion           },
   { "auto-select",         &fc_autoselect               },
   { "bad-path",            &fc_badpath                  },
+  { "not-root",            &fc_nonroot                  },
   { "overwrite",           &fc_overwrite                },
   { "overwrite-diverted",  &fc_overwritediverted        },
+  { "overwrite-dir",       &fc_overwritedir             },
   { "architecture",        &fc_architecture             },
   {  0                                                  }
 };
@@ -243,12 +249,14 @@ DPKG " forcing options - control behaviour when problems found:\n\
   configure-any          Configure any package which may help this one\n\
   hold                   Process incidental packages even when on hold\n\
   bad-path               PATH is missing important programs, problems likely\n\
+  not-root               Try to (de)install things even when not root\n\
   overwrite [*]          Overwrite a file from one package with another\n\
   overwrite-diverted     Overwrite a diverted file with an undiverted version\n\
   depends-version [!]    Turn dependency version problems into warnings\n\
   depends [!]            Turn all dependency problems into warnings\n\
   conflicts [!]          Allow installation of conflicting packages\n\
   architecture [!]       Process even packages with wrong architecture\n\
+  overwrite-dir [!]      Overwrite one package's directory with another's file\n\
   remove-reinstreq [!]   Remove packages which require installation\n\
   remove-essential [!]   Remove an essential package\n\
 \n\
@@ -294,6 +302,8 @@ static const struct cmdinfo cmdinfos[]= {
   ACTION( "purge",                           0,  act_purge,         packages        ),
   ACTION( "listfiles",                      'L', act_listfiles,     enqperpackage   ),
   ACTION( "status",                         's', act_status,        enqperpackage   ),
+  ACTION( "get-selections",                  0,  act_getselections, getselections   ),
+  ACTION( "set-selections",                  0,  act_setselections, setselections   ),
   ACTION( "print-avail",                     0,  act_printavail,    enqperpackage   ),
   ACTION( "update-avail",                    0,  act_avreplace,     updateavailable ),
   ACTION( "merge-avail",                     0,  act_avmerge,       updateavailable ),
@@ -304,6 +314,7 @@ static const struct cmdinfo cmdinfos[]= {
   ACTION( "list",                           'l', act_listpackages,  listpackages    ),
   ACTION( "search",                         'S', act_searchfiles,   searchfiles     ),
   ACTION( "print-architecture",              0,  act_printarch,     printarch       ),
+  ACTION( "print-gnu-build-architecture",    0,  act_printgnuarch,  printarch       ),
   ACTION( "assert-support-predepends",       0,  act_assertpredep,  assertpredep    ),
   ACTION( "print-installation-architecture", 0,  act_printinstarch, printinstarch   ),
   ACTION( "predep-package",                  0,  act_predeppackage, predeppackage   ),
